## This function reads the output generated by the function spider_stats.stats_log() and generates a figure showing
## the timeline of the executed jobs and the memory usage

# INPUT DATA (file log_stats.txt)
args  <- commandArgs(trailingOnly = TRUE)
lpath <- args[1]
steps <- 100 # time resolution for the figure
# READS DATA AND PARSES DATETIME COLUMNS (STARTING AND ENDING TIMES)
log  <- read.table(lpath, header = T, sep = "\t", stringsAsFactors = F)
runs <- sort(unique(log[,1]))
dt   <- matrix(NA, nrow = nrow(log), ncol = 2)
for (i in 1:nrow(log)){
  for (j in c(6,7)){
    k  <- strsplit(log[i,j], " ", fixed = T)[[1]]
    dt[i,j-5]  <- as.POSIXct(paste(k[2:length(k)], collapse = " "), format = "%b %d %H:%M:%S %Y")
  }
}
# EXTRACTS THE DIFFERENT TYPE OF JOBS AND ASSIGNS THE X POSITION WHERE LABELS WILL BE PLOTTED
# GENERATES A TIMELINE PLOT FOR EACH AVAILABLE RUN
for (irun in 1:length(runs)){
  j <- which(log[,1] == runs[irun])
  lab   <- log[j,3]
  n     <- c()
  for (i in 2:length(lab)){
    if (lab[i-1] != lab[i]){
      n <- c(n, i - 0.5)
    }
  }
  xx  <- c()
  for (i in unique(lab)){
    xx <- c(xx,mean(which(lab == i)))
  }
  a <- png(gsub(".txt", paste("_", irun, ".png", sep = ""), lpath, fixed = T), width = 800, height = 600, res = 300, bg = NA, pointsize = 6)
  x <- seq(min(dt[j,1], na.rm = T),max(dt[j,2], na.rm = T),length.out = steps)
  M <- matrix(0, nrow = 3 * length(j), ncol = length(x))
  for (kt in 1:length(j)){
    if ((is.na(dt[j[kt],1]) == F) & (is.na(dt[j[kt],2]) == F)){
      i <- which((x >= dt[j[kt],1]) & (x <= dt[j[kt],2]))
      M[kt*3-2, i] <- log10(log[j[kt], 9])
      M[kt*3-1, i] <- log10(log[j[kt], 10])
      M[kt*3  , i] <- log10(log[j[kt], 11])
    }
  }
  par(mar = c(8, 4, 1, 1))
  image((M), col = heat.colors(20), axes = F)
  y <- seq(0, 1, length.out = nrow(M))
  y <- y - diff(y[1:2])/2
  abline(v = y[seq(1, length(y), by = 3)], col ="Gray", lwd = 0.4)
  abline(v = y[n * 3], col = "black", lwd = 0.8)
  axis(1, at = y[round(xx * 3)], labels = unique(lab), las = 2)
  axis(2)
  mtext(paste("Total run: ", round((max(dt[j,]) - min(dt[j,])) / 3600, 2), " hours"), side = 2, line = 2)
  a <- dev.off()
}
